@@ -81,7 +81,7 @@ def parse(source: Union[str, bytes],
           fnam: str,
           module: Optional[str],
           errors: Optional[Errors] = None,
-          options: Options = Options()) -> MypyFile:
+          options: Optional[Options] = None) -> MypyFile:
     """Parse a source file, without doing any semantic analysis.
 
     Return the parse tree. If errors is not provided, raise ParseError
@@ -91,6 +91,8 @@ def parse(source: Union[str, bytes],
     if errors is None:
         errors = Errors()
         raise_on_error = True
+    if options is None:
+        options = Options()
     errors.set_file(fnam, module)
     is_stub_file = fnam.endswith('.pyi')
     try:
@@ -910,8 +912,12 @@ class ASTConverter(ast27.NodeTransformer):
 
     # List(expr* elts, expr_context ctx)
     @with_line
-    def visit_List(self, n: ast27.List) -> ListExpr:
-        return ListExpr([self.visit(e) for e in n.elts])
+    def visit_List(self, n: ast27.List) -> Union[ListExpr, TupleExpr]:
+        expr_list = [self.visit(e) for e in n.elts]  # type: List[Expression]
+        if isinstance(n.ctx, ast27.Store):
+            # [x, y] = z and (x, y) = z means exactly the same thing
+            return TupleExpr(expr_list)
+        return ListExpr(expr_list)
 
     # Tuple(expr* elts, expr_context ctx)
     @with_line
