@@ -10,7 +10,7 @@ from mypy.nodes import (
     GeneratorExpr, ListComprehension, SetComprehension, DictionaryComprehension,
     ConditionalExpr, TypeApplication, ExecStmt, Import, ImportFrom,
     LambdaExpr, ComparisonExpr, OverloadedFuncDef, YieldFromExpr,
-    YieldExpr, StarExpr, BackquoteExpr, AwaitExpr, PrintStmt,
+    YieldExpr, StarExpr, BackquoteExpr, AwaitExpr, PrintStmt, SuperExpr,
 )
 
 
@@ -34,13 +34,14 @@ class TraverserVisitor(NodeVisitor[None]):
             s.accept(self)
 
     def visit_func(self, o: FuncItem) -> None:
-        for arg in o.arguments:
-            init = arg.initializer
-            if init is not None:
-                init.accept(self)
+        if o.arguments is not None:
+            for arg in o.arguments:
+                init = arg.initializer
+                if init is not None:
+                    init.accept(self)
 
-        for arg in o.arguments:
-            self.visit_var(arg.variable)
+            for arg in o.arguments:
+                self.visit_var(arg.variable)
 
         o.body.accept(self)
 
@@ -59,6 +60,8 @@ class TraverserVisitor(NodeVisitor[None]):
         for base in o.base_type_exprs:
             base.accept(self)
         o.defs.accept(self)
+        if o.analyzed:
+            o.analyzed.accept(self)
 
     def visit_decorator(self, o: Decorator) -> None:
         o.func.accept(self)
@@ -126,6 +129,9 @@ class TraverserVisitor(NodeVisitor[None]):
             if tp is not None:
                 tp.accept(self)
             o.handlers[i].accept(self)
+        for v in o.vars:
+            if v is not None:
+                v.accept(self)
         if o.else_body is not None:
             o.else_body.accept(self)
         if o.finally_body is not None:
@@ -250,6 +256,9 @@ class TraverserVisitor(NodeVisitor[None]):
     def visit_await_expr(self, o: AwaitExpr) -> None:
         o.expr.accept(self)
 
+    def visit_super_expr(self, o: SuperExpr) -> None:
+        o.call.accept(self)
+
     def visit_import(self, o: Import) -> None:
         for a in o.assignments:
             a.accept(self)
